@@ -44,12 +44,19 @@
 
 /* Please increase version number with every change 
    (don't forget to update dll.desc) */
-#define DLL_VERSION "1.0.10"
+#define DLL_VERSION "1.0.12"
 
 #ifdef _AIX
 # include "lalloca.h"		/* MUST come first for AIX! */
 #endif
 
+#ifdef __BEOS__
+#include <kernel/OS.h>
+#include <storage/FindDirectory.h>
+#include <kernel/image.h>
+#include <posix/dirent.h>
+#endif
+
 #include "sane/config.h"
 #include "lalloca.h"
 
@@ -99,6 +106,17 @@
 # define PATH_MAX       1024
 #endif
 
+#if defined(HAVE_OS2_H)
+# define DIR_SEP        ";"
+#elif defined(__CYGWIN__)
+# define DIR_SEP        ":"
+#elif defined(HAVE_WINDOWS_H)
+# define DIR_SEP        ";"
+#else
+# define DIR_SEP        ":"
+#endif
+
+
 #include "sane/sanei_config.h"
 #define DLL_CONFIG_FILE "dll.conf"
 #define DLL_ALIASES_FILE "dll.aliases"
@@ -190,7 +208,9 @@ struct backend
   }                                             \
 }
 
+#ifndef __BEOS__
 #include "dll-preload.c"
+#endif
 
 struct meta_scanner
 {
@@ -215,11 +235,20 @@ static int devlist_size = 0, devlist_len = 0;
 static struct alias *first_alias;
 static SANE_Auth_Callback auth_callback;
 static struct backend *first_backend;
+
+#ifndef __BEOS__
 static const char *op_name[] = {
   "init", "exit", "get_devices", "open", "close", "get_option_descriptor",
   "control_option", "get_parameters", "start", "read", "cancel",
   "set_io_mode", "get_select_fd"
 };
+#else
+static const char *op_name[] = {
+  "sane_init", "sane_exit", "sane_get_devices", "sane_open", "sane_close", "sane_get_option_descriptor",
+  "sane_control_option", "sane_get_parameters", "sane_start", "sane_read", "sane_cancel",
+  "sane_set_io_mode", "sane_get_select_fd"
+};
+#endif /* __BEOS__ */
 
 static void *
 op_unsupported (void)
@@ -289,6 +318,57 @@ dyld_get_error_str ()
 }
 #endif
 
+#ifdef __BEOS__
+#include <FindDirectory.h>
+
+static SANE_Status
+load (struct backend *be)
+{
+	/* use BeOS kernel function to load scanner addons from ~/config/add-ons/SANE */
+	char path[PATH_MAX];
+	image_id id = -1;
+	int i, w;
+	directory_which which[3] = { B_USER_ADDONS_DIRECTORY, B_COMMON_ADDONS_DIRECTORY, B_BEOS_ADDONS_DIRECTORY };
+	
+	/* look for config files in SANE/conf */
+	for (w = 0; (w < 3) && (id < 0) && (find_directory(which[w],0,true,path,PATH_MAX) == 0); w++)
+	{
+		strcat(path,"/SANE/");
+		strcat(path,be->name);
+		DBG(1, "loading backend %s\n", be->name);
+
+		/* initialize all ops to "unsupported" so we can "use" the backend
+     	   even if the stuff later in this function fails */
+		be->loaded = 1;
+		be->handle = 0;
+		for (i = 0; i < NUM_OPS; ++i) be->op[i] = op_unsupported;
+  		DBG(2, "dlopen()ing `%s'\n", path);
+		id=load_add_on(path);
+		if (id < 0)
+		{
+			continue; /* try next path */
+		}
+    	be->handle=(void *)id;
+	    	
+		for (i = 0; i < NUM_OPS; ++i)
+    	{
+      		void *(*op) ();
+      		op = NULL;
+	      	/* Look for the symbol */
+			if ((get_image_symbol(id, op_name[i],B_SYMBOL_TYPE_TEXT,(void **)&op) < 0) || !op)
+			    DBG(2, "unable to find %s\n", op_name[i]);
+      		else be->op[i]=op;
+      	}
+    }
+	if (id < 0)
+   	{
+		DBG(2, "load: couldn't find %s\n",path);
+     	return SANE_STATUS_INVAL;
+   	}
+  return SANE_STATUS_GOOD;
+}
+
+#else
 static SANE_Status
 load (struct backend *be)
 {
@@ -304,6 +384,7 @@ load (struct backend *be)
 # define PREFIX "libsane-"
 # ifdef __hpux
 #   define POSTFIX ".sl.%u"
+#   define ALT_POSTFIX ".so.%u"
 #elif defined (HAVE_WINDOWS_H)
 #   undef PREFIX
 #   define PREFIX "cygsane-"
@@ -365,19 +446,37 @@ load (struct backend *be)
     }
   DBG (3, "load: searching backend `%s' in `%s'\n", be->name, src);
 
-  dir = strsep (&src, ":");
+  dir = strsep (&src, DIR_SEP);
 
   while (dir)
     {
+#ifdef HAVE_OS2_H   /* only max 7.3 names work with dlopen() for DLLs on OS/2 */
+      snprintf (libname, sizeof (libname), "%s/" PREFIX "%.2s%.5s" POSTFIX,
+		dir, be->name, strlen(be->name)>7 ? (be->name)+strlen(be->name)-5 :
+                                            (be->name)+2, V_MAJOR);
+#else
       snprintf (libname, sizeof (libname), "%s/" PREFIX "%s" POSTFIX,
 		dir, be->name, V_MAJOR);
+#endif
       DBG (4, "load: trying to load `%s'\n", libname);
       fp = fopen (libname, "r");
       if (fp)
 	break;
       DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
 
-      dir = strsep (&src, ":");
+#ifdef ALT_POSTFIX      
+      /* Some platforms have two ways of storing their libraries, try both
+	 postfixes */
+      snprintf (libname, sizeof (libname), "%s/" PREFIX "%s" ALT_POSTFIX,
+		dir, be->name, V_MAJOR);
+      DBG (4, "load: trying to load `%s'\n", libname);
+      fp = fopen (libname, "r");
+      if (fp)
+	break;
+      DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
+#endif
+
+      dir = strsep (&src, DIR_SEP);
     }
   if (orig_src)
     free (orig_src);
@@ -490,6 +589,7 @@ load (struct backend *be)
   return SANE_STATUS_UNSUPPORTED;
 #endif /* HAVE_DLL */
 }
+#endif /* __BEOS__ */
 
 static SANE_Status
 init (struct backend *be)
@@ -530,6 +630,7 @@ init (struct backend *be)
 static void
 add_alias (const char *line_param)
 {
+#ifndef __BEOS__
   const char *command;
   enum
   { CMD_ALIAS, CMD_HIDE }
@@ -613,17 +714,26 @@ add_alias (const char *line_param)
       free (alias);
     }
   return;
+#endif
 }
 
 
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
+#ifndef __BEOS__
   char config_line[PATH_MAX];
   char *backend_name;
   size_t len;
   FILE *fp;
   int i;
+#else
+  DIR *dir;
+  struct dirent *dirent;
+  char path[1024];
+  directory_which which[3] = { B_USER_ADDONS_DIRECTORY, B_COMMON_ADDONS_DIRECTORY, B_BEOS_ADDONS_DIRECTORY };
+  int i;
+#endif	
 
   DBG_INIT ();
 
@@ -632,6 +742,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (1, "sane_init: SANE dll backend version %s from %s\n", DLL_VERSION,
        PACKAGE_STRING);
 
+#ifndef __BEOS__
   /* chain preloaded backends together: */
   for (i = 0; i < NELEMS (preloaded_backends); ++i)
     {
@@ -703,6 +814,28 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       add_alias (config_line);
     }
   fclose (fp);
+
+#else  
+	/* no ugly config files, just get scanners from their ~/config/add-ons/SANE */
+	/* look for drivers */
+	for (i = 0; i < 3; i++)
+	{
+		if (find_directory(which[i],0,true,path,1024) < B_OK)
+			continue;
+		strcat(path,"/SANE/");
+		dir=opendir(path);
+		if(!dir) continue; 
+
+		while((dirent=readdir(dir)))
+		{
+			if((strcmp(dirent->d_name,".")==0) || (strcmp(dirent->d_name,"..")==0)) continue;
+			if((strcmp(dirent->d_name,"dll")==0)) continue;
+			add_backend(dirent->d_name,0); 
+		}
+		closedir(dir);
+	}
+#endif /* __BEOS__ */
+
   return SANE_STATUS_GOOD;
 }
 
@@ -725,6 +858,10 @@ sane_exit (void)
 		   be->name);
 	      (*(op_exit_t)be->op[OP_EXIT]) ();
 	    }
+#ifdef __BEOS__
+	  /* use BeOS kernel functions to unload add-ons */
+	  if(be->handle) unload_add_on((image_id)be->handle);
+#else
 #ifdef HAVE_DLL
 
 #ifdef HAVE_DLOPEN
@@ -745,6 +882,7 @@ sane_exit (void)
 #endif /* HAVE_DLOPEN */
 
 #endif /* HAVE_DLL */
+#endif /* __BEOS__ */
 	}
       if (!be->permanent)
 	{
