@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -62,35 +62,6 @@
 #undef MAX_DEBUG
 #endif
 
-/** Declare @c function_name if debugging is enabled.
- *
- * @param name Function name (should be a string constant).
- *
- * This macro should be used only at the beginning of a function body.  If
- * debugging is enabled (#NDEBUG is not defined), this macro declares a static
- * variable @c function_name, initialized with @name.  If debugging is disabled
- * (#NDEBUG is defined), this macro expands to nothing.
- *
- * @note When using this macro, @c function_name must be used only if debugging
- * is enabled.  In particular, @c function_name cannot be used with the #DBG
- * macro directly, because the arguments of #DBG are evaluated even if
- * debugging is disabled.  Use #XDBG(args) in this case.
- */
-#define DECLARE_FUNCTION_NAME(name)     \
-  IF_DBG ( static const char function_name[] = name; )
-
-/** Print a debug message if the debug level is sufficiently high.
- *
- * If the debugging is enabled (#NDEBUG is not defined), this macro is almost
- * equivalent to #DBG, except that you need an extra pair of parentheses around
- * all arguments.  If the debugging is disabled (#NDEBUG is defined), this
- * macro expands to an empty operator.
- * 
- * @note The arguments of this macro must not have side effects - they are not
- * evaluated when debugging is disabled.
- */
-#define XDBG(args)           do { IF_DBG ( DBG args ); } while (0)
-
 /* calculate the minimum/maximum values */
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
@@ -101,13 +72,13 @@
 
 #define MM_PER_INCH 25.4
 
+/* return if an error occured while the function was called */
 #ifdef MAX_DEBUG
+#  ifndef __FUNCTION__
+#    define __FUNCTION__ "somewhere"
+#  endif
 
-#ifndef __FUNCTION__
-#  define __FUNCTION__ "somewhere"
-#endif
-
-#define RIE(function) \
+#  define RIE(function) \
   do \
     { \
       status = function; \
@@ -122,7 +93,7 @@
 
 #else
 
-#define RIE(function)                                   \
+#  define RIE(function)                                   \
   do { status = function;                               \
     if (status != SANE_STATUS_GOOD) return status;      \
   } while (SANE_FALSE)
@@ -130,12 +101,19 @@
 #endif
 
 /* Flags */
-#define GT68XX_FLAG_MIRROR_X	 (1 << 0)	/* CIS unit mounted the other way round? */
-#define GT68XX_FLAG_MOTOR_HOME	 (1 << 1)	/* Use motor_home command (0x34) */
-#define GT68XX_FLAG_OFFSET_INV   (1 << 2)	/* Offset control is inverted */
-#define GT68XX_FLAG_UNTESTED     (1 << 3)	/* Print a warning for these scanners */
-#define GT68XX_FLAG_SE_2400      (1 << 4)	/* Special quirks for SE 2400USB */
-#define GT68XX_FLAG_NO_STOP      (1 << 5)	/* Don't call stop_scan before the scan */
+#define GT68XX_FLAG_MIRROR_X	    (1 << 0)	/* CIS unit mounted the other way round? */
+#define GT68XX_FLAG_MOTOR_HOME	    (1 << 1)	/* Use motor_home command (0x34) */
+#define GT68XX_FLAG_OFFSET_INV      (1 << 2)	/* Offset control is inverted */
+#define GT68XX_FLAG_UNTESTED        (1 << 3)	/* Print a warning for these scanners */
+#define GT68XX_FLAG_SE_2400         (1 << 4)	/* Special quirks for SE 2400USB */
+#define GT68XX_FLAG_NO_STOP         (1 << 5)	/* Don't call stop_scan before the scan */
+#define GT68XX_FLAG_CIS_LAMP        (1 << 6)	/* CIS sensor with lamp */
+#define GT68XX_FLAG_NO_POWER_STATUS (1 << 7)	/* get_power_status_doesn't work */
+#define GT68XX_FLAG_NO_LINEMODE     (1 << 8)	/* Linemode does not work with this scanner */
+#define GT68XX_FLAG_SCAN_FROM_HOME  (1 << 9)	/* Move home after calibration */
+#define GT68XX_FLAG_USE_OPTICAL_X   (1 << 10)	/* Use optical xdpi for 50 dpi and below */
+#define GT68XX_FLAG_ALWAYS_LINEMODE (1 << 11)	/* Linemode must be used for any resolution */
+#define GT68XX_FLAG_SHEET_FED       (1 << 12)	/* we have a sheet fed scanner */
 
 /* Forward typedefs */
 typedef struct GT68xx_USB_Device_Entry GT68xx_USB_Device_Entry;
@@ -154,6 +132,10 @@ typedef enum GT68xx_Color_Order
 }
 GT68xx_Color_Order;
 
+#define GT68XX_COLOR_RED SANE_I18N ("Red")
+#define GT68XX_COLOR_GREEN SANE_I18N ("Green")
+#define GT68XX_COLOR_BLUE SANE_I18N ("Blue")
+
 /** Scan action code (purpose of the scan).
  *
  * The scan action code affects various scanning mode fields in the setup
@@ -176,7 +158,7 @@ struct GT68xx_USB_Device_Entry
   GT68xx_Model *model;			/**< Scanner model information */
 };
 
-#define MAX_SCANNERS 30
+#define MAX_SCANNERS 50
 /* Looks like gcc doesn't like declarations without specificating the number
    of array elements at least when --enable-warnings is active */
 
@@ -352,6 +334,7 @@ struct GT68xx_Command_Set
    */
     SANE_Status (*is_moving) (GT68xx_Device * dev, SANE_Bool * moving);
 
+
   /** Move the scanner carriage by the specified number of steps.
    *
    * @param dev Device object.
@@ -372,6 +355,12 @@ struct GT68xx_Command_Set
    */
     SANE_Status (*carriage_home) (GT68xx_Device * dev);
 
+  /** Eject the paper at the end of the scan.
+   *
+   * @param dev Device object.
+   */
+    SANE_Status (*paperfeed) (GT68xx_Device * dev);
+
   /** Start scanning the image.
    *
    * @param dev Device object.
@@ -440,26 +429,26 @@ struct GT68xx_Model
 
   GT68xx_Command_Set *command_set;
 
-  SANE_Int optical_xdpi;	/* maximum resolution in x-direction */	      
-  SANE_Int optical_ydpi;	/* maximum resolution in y-direction */	      
-  SANE_Int base_xdpi;		/* x-resolution used to calculate geometry */ 
-  SANE_Int base_ydpi;		/* y-resolution used to calculate geometry */ 
-  SANE_Int ydpi_force_line_mode;/* if ydpi is equal or higher, use linemode */
+  SANE_Int optical_xdpi;	/* maximum resolution in x-direction */
+  SANE_Int optical_ydpi;	/* maximum resolution in y-direction */
+  SANE_Int base_xdpi;		/* x-resolution used to calculate geometry */
+  SANE_Int base_ydpi;		/* y-resolution used to calculate geometry */
+  SANE_Int ydpi_no_backtrack;	/* if ydpi is equal or higher, disable backtracking */
   SANE_Bool constant_ydpi;	/* Use base_ydpi for all resolutions        */
 
-  SANE_Int xdpi_values[MAX_RESOLUTIONS]; /* possible x resolutions */	 
-  SANE_Int ydpi_values[MAX_RESOLUTIONS]; /* possible y resolutions */	 
-  SANE_Int bpp_gray_values[MAX_DPI]; /* possible depths in gray mode */	 
-  SANE_Int bpp_color_values[MAX_DPI]; /* possible depths in color mode */
+  SANE_Int xdpi_values[MAX_RESOLUTIONS];	/* possible x resolutions */
+  SANE_Int ydpi_values[MAX_RESOLUTIONS];	/* possible y resolutions */
+  SANE_Int bpp_gray_values[MAX_DPI];	/* possible depths in gray mode */
+  SANE_Int bpp_color_values[MAX_DPI];	/* possible depths in color mode */
+
+  SANE_Fixed x_offset;		/* Start of scan area in mm */
+  SANE_Fixed y_offset;		/* Start of scan area in mm */
+  SANE_Fixed x_size;		/* Size of scan area in mm */
+  SANE_Fixed y_size;		/* Size of scan area in mm */
+
+  SANE_Fixed y_offset_calib;	/* Start of white strip in mm */
+  SANE_Fixed x_offset_mark;	/* Start of black mark in mm */
 
-  SANE_Fixed x_offset;		/* Start of scan area in mm */		
-  SANE_Fixed y_offset;		/* Start of scan area in mm */		
-  SANE_Fixed x_size;		/* Size of scan area in mm */		
-  SANE_Fixed y_size;		/* Size of scan area in mm */		
-				                                        
-  SANE_Fixed y_offset_calib;	/* Start of white strip in mm */	
-  SANE_Fixed x_offset_mark;	/* Start of black mark in mm */		
-				                                        
   SANE_Fixed x_offset_ta;	/* Start of scan area in TA mode in mm */
   SANE_Fixed y_offset_ta;	/* Start of scan area in TA mode in mm */
   SANE_Fixed x_size_ta;		/* Size of scan area in TA mode in mm */
@@ -468,17 +457,17 @@ struct GT68xx_Model
   SANE_Fixed y_offset_calib_ta;	/* Start of white strip in TA mode in mm */
 
   /* Line-distance correction (in pixel at optical_ydpi) for CCD scanners */
-  SANE_Int ld_shift_r; /* red */
-  SANE_Int ld_shift_g; /* green */
-  SANE_Int ld_shift_b; /* blue */
-  SANE_Int ld_shift_double; /* distance between two CCD lines of one color
-			       (only applicable for CCD with 6 lines) */
+  SANE_Int ld_shift_r;		/* red */
+  SANE_Int ld_shift_g;		/* green */
+  SANE_Int ld_shift_b;		/* blue */
+  SANE_Int ld_shift_double;	/* distance between two CCD lines of one color
+				   (only applicable for CCD with 6 lines) */
 
-  GT68xx_Color_Order line_mode_color_order; /* Order of the CCD/CIS colors */
+  GT68xx_Color_Order line_mode_color_order;	/* Order of the CCD/CIS colors */
 
-  GT68xx_AFE_Parameters afe_params; /* Default offset/gain */
-  GT68xx_Exposure_Parameters exposure; /* Default exposure parameters */
-  SANE_Fixed default_gamma_value; /* Default gamma value */
+  GT68xx_AFE_Parameters afe_params;	/* Default offset/gain */
+  GT68xx_Exposure_Parameters exposure;	/* Default exposure parameters */
+  SANE_Fixed default_gamma_value;	/* Default gamma value */
 
   SANE_Bool is_cis;		/* Is this a CIS or CCD scanner? */
 
@@ -515,7 +504,8 @@ struct GT68xx_Device
   size_t read_pos;
   size_t read_bytes_in_buffer;
   size_t read_bytes_left;
-
+  SANE_Byte gray_mode_color;
+  SANE_Bool manual_selection;
 #ifdef USE_FORK
   Shm_Channel *shm_channel;
   pid_t reader_pid;
@@ -548,15 +538,15 @@ struct GT68xx_Scan_Request
   SANE_Bool lamp;	/**< Lamp on/off */
   SANE_Bool calculate;	/**< Don't scan, only calculate parameters */
   SANE_Bool use_ta;	/**< Use the tansparency adapter */
+  SANE_Bool backtrack;	/**< Enable backtracking */
+  SANE_Bool backtrack_lines;  /**< How many lines to backtrack */
 };
 
 /** Scan parameters for gt68xx_device_setup_scan().
  *
  * These parameters describe a low-level scan request; many such requests are
  * executed during calibration, and they need to have parameters separate from
- * the main request (GT68xx_Scan_Request).  E.g., on the BearPaw 2400 TA the
- * scan to find the home position is always done at 300dpi 8-bit mono with
- * fixed width and height, regardless of the high-level scan parameters.
+ * the main request (GT68xx_Scan_Request).  
  */
 struct GT68xx_Scan_Parameters
 {
@@ -807,6 +797,7 @@ static SANE_Status
 gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
 			 GT68xx_Packet res);
 
+#if 0
 /** Check whether the firmware is downloaded into the scanner.
  *
  * @param dev Device object.
@@ -820,6 +811,7 @@ gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
  */
 static SANE_Status
 gt68xx_device_check_firmware (GT68xx_Device * dev, SANE_Bool * loaded);
+#endif
 
 static SANE_Status
 gt68xx_device_download_firmware (GT68xx_Device * dev,
@@ -919,6 +911,17 @@ gt68xx_device_move_relative (GT68xx_Device * dev, SANE_Int distance);
  */
 static SANE_Status gt68xx_device_carriage_home (GT68xx_Device * dev);
 
+/** Eject the paper after the end of scanning.
+ *
+ *
+ * @param dev Device object.
+ *
+ * @return
+ * - #SANE_STATUS_GOOD - success; the movement is started.
+ * - #SANE_STATUS_IO_ERROR - a communication error occured.
+ */
+static SANE_Status gt68xx_device_paperfeed (GT68xx_Device * dev);
+
 /** Start scanning the image.
  *
  * This function initiates scanning with parameters set by
@@ -1054,7 +1057,20 @@ static SANE_Status
 gt68xx_device_check_result (GT68xx_Packet res, SANE_Byte command);
 
 
-static SANE_Status gt68xx_device_get_id (GT68xx_Device * dev);
+static SANE_Status 
+gt68xx_device_get_id (GT68xx_Device * dev);
+
+/** Read the device descriptor of the scanner.
+ *
+ * This function should be called before closing the device to make sure
+ * that the device descriptor is propperly stored in the scanner's memory.
+ * If that's not done, the next try to get the config descriptor will
+ * result in a corrupted descriptor.
+ *
+ * @param dev device
+*/
+static void 
+gt68xx_device_fix_descriptor (GT68xx_Device * dev);
 
 #endif /* not GT68XX_LOW_H */
 
