@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002, 2004 Frank Zago (sane at zago dot net)
    Copyright (C) 2002 Other SANE contributors
    
    This file is part of the SANE package.
@@ -43,16 +43,16 @@
 */
 
 /*
-   $Id: matsushita.c,v 1.6 2002/11/19 05:01:23 fzago Exp $
+   $Id: matsushita.c,v 1.10 2005/09/19 19:57:48 fzago-guest Exp $
    Matsushita/Panasonic KV-SS25, KV-SS50, KV-SS55, KV-SS50EX, 
-                        KV-SS55EX, KV-SS850, KV-SS855 SCSI scanner.
+                        KV-SS55EX, KV-SS850, KV-SS855 SCSI scanners.
 
    This backend may support more Panasonic scanners.
 */
 
 /*--------------------------------------------------------------------------*/
 
-#define BUILD 6			/* 2002-03-13 */
+#define BUILD 7			/* 2004-02-11 */
 #define BACKEND_NAME matsushita
 #define MATSUSHITA_CONFIG_FILE "matsushita.conf"
 
@@ -380,17 +380,17 @@ static const struct scanners_supported scanners[] = {
 
   /* Panasonic KV-SS55EX */
   {
-   0x06, "K.M.E.  ", "KV-SS55EX       ",	/* TO FIX */
+   0x06, "K.M.E.  ", "KV-SS55EX       ",
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},	/* x range 0 to 215.9 mm */
-   {SANE_FIX (0), SANE_FIX (17 * MM_PER_INCH), 0},	/* y range 0 to 355.6 mm */
+   {SANE_FIX (0), SANE_FIX (17 * MM_PER_INCH), 0},	/* y range 0 to 431.8 mm */
    {1, 255, 1},			/* brightness range */
-   {1, 255, 1},			/* contrast range, TO FIX */
+   {1, 255, 1},			/* contrast range */
    scan_mode_list_3,
-   resolutions_list_300, resolutions_rounds_300,	/* TO FIX */
+   resolutions_list_300, resolutions_rounds_300,
    image_emphasis_list_5, image_emphasis_val_5,
-   MAT_CAP_DUPLEX | MAT_CAP_CONTRAST | MAT_CAP_AUTOMATIC_THRESHOLD |
-   MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA | MAT_CAP_NOISE_REDUCTION |
-   MAT_CAP_PAPER_DETECT | MAT_CAP_MIRROR_IMAGE},
+   MAT_CAP_DUPLEX | MAT_CAP_CONTRAST |
+   MAT_CAP_AUTOMATIC_THRESHOLD | MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA |
+   MAT_CAP_NOISE_REDUCTION},
 
   /* Panasonic KV-SS850 */
   {
@@ -418,7 +418,49 @@ static const struct scanners_supported scanners[] = {
    image_emphasis_list_5, image_emphasis_val_5,
    MAT_CAP_DUPLEX | MAT_CAP_CONTRAST | MAT_CAP_AUTOMATIC_THRESHOLD |
    MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA | MAT_CAP_NOISE_REDUCTION |
-   MAT_CAP_PAPER_DETECT | MAT_CAP_DETECT_DOUBLE_FEED | MAT_CAP_MANUAL_FEED}
+   MAT_CAP_PAPER_DETECT | MAT_CAP_DETECT_DOUBLE_FEED | MAT_CAP_MANUAL_FEED},
+
+  /* Panasonic KV-S2065L */
+  {
+   0x06, "K.M.E.  ", "KV-S2065L       ",
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},	/* x range 0 to 215.9 mm */
+   {SANE_FIX (0), SANE_FIX (17 * MM_PER_INCH), 0},	/* y range 0 to 431.8 mm */
+   {1, 255, 1},			/* brightness range */
+   {1, 255, 1},			/* contrast range */
+   scan_mode_list_3,
+   resolutions_list_300, resolutions_rounds_300,
+   image_emphasis_list_5, image_emphasis_val_5,
+   MAT_CAP_DUPLEX | MAT_CAP_CONTRAST |
+   MAT_CAP_AUTOMATIC_THRESHOLD | MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA |
+   MAT_CAP_NOISE_REDUCTION},
+
+  /* Panasonic KV-S2025C */
+  {
+   0x06, "K.M.E.  ", "KV-S2025C       ",
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},	/* x range 0 to 215.9 mm */
+   {SANE_FIX (0), SANE_FIX (17 * MM_PER_INCH), 0},	/* y range 0 to 431.8 mm */
+   {1, 255, 1},			/* brightness range */
+   {1, 255, 1},			/* contrast range */
+   scan_mode_list_3,
+   resolutions_list_300, resolutions_rounds_300,
+   image_emphasis_list_5, image_emphasis_val_5,
+   MAT_CAP_DUPLEX | MAT_CAP_CONTRAST |
+   MAT_CAP_AUTOMATIC_THRESHOLD | MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA |
+   MAT_CAP_NOISE_REDUCTION},
+
+  /* Panasonic KV-S2045C */
+  {
+   0x06, "K.M.E.  ", "KV-S2045C       ",
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},	/* x range 0 to 215.9 mm */
+   {SANE_FIX (0), SANE_FIX (17 * MM_PER_INCH), 0},	/* y range 0 to 431.8 mm */
+   {1, 255, 1},			/* brightness range */
+   {1, 255, 1},			/* contrast range */
+   scan_mode_list_3,
+   resolutions_list_300, resolutions_rounds_300,
+   image_emphasis_list_5, image_emphasis_val_5,
+   MAT_CAP_DUPLEX | MAT_CAP_CONTRAST |
+   MAT_CAP_AUTOMATIC_THRESHOLD | MAT_CAP_WHITE_LEVEL | MAT_CAP_GAMMA |
+   MAT_CAP_NOISE_REDUCTION}
 };
 
 
@@ -798,15 +840,13 @@ get_int_list_index (const SANE_Word list[], const SANE_Word value)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-matsushita_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+matsushita_sense_handler (int scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
 
   DBG (DBG_proc, "matsushita_sense_handler (scsi_fd = %d)\n", scsi_fd);
 
-  arg = arg;			/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1663,7 +1703,7 @@ do_cancel (Matsushita_Scanner * dev)
 /* Entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1673,8 +1713,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-matsushita version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1712,15 +1750,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Matsushita_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2376,16 +2412,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
 	SANE_Status status;
 	Matsushita_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
     if (dev->scanning == SANE_FALSE)
     {
       return (SANE_STATUS_INVAL);
@@ -2403,13 +2436,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
