@@ -68,35 +68,25 @@
      So far this strategy has worked flawlessly. Thanks Dmitri!
 */
 
-/* $Id: snapscan-usb.c,v 1.13 2003/11/08 09:50:27 oliver-guest Exp $
+/* $Id: snapscan-usb.c,v 1.21 2005/11/02 19:22:06 oliver-guest Exp $
    SnapScan backend scan data sources */
 
-#include <sys/ipc.h>
-#include <sys/sem.h>
-
 #include "snapscan-usb.h"
-
-/* check for union semun */
-#if defined(HAVE_UNION_SEMUN)
-/* union semun is defined by including <sys/sem.h> */
-#else
-/* according to X/OPEN we have to define it ourselves */
-union semun {
-   int val;                    /* value for SETVAL */
-   struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
-   unsigned short int *array;  /* array for GETALL, SETALL */
-   struct seminfo *__buf;      /* buffer for IPC_INFO */
-};
-#endif
+#include "snapscan-mutex.c"
 
 /* Global variables */
 
-static int sem_id;
-static struct sembuf sem_wait = { 0, -1, 0 };
-static struct sembuf sem_signal = { 0, 1, 0 };
+static snapscan_mutex_t snapscan_mutex;
 static sense_handler_type usb_sense_handler;
 static void* usb_pss;
 
+struct urb_counters_t {
+    unsigned long read_urbs;
+    unsigned long write_urbs;
+};
+
+static struct urb_counters_t* urb_counters = NULL;
+
 /* Forward declarations */
 static SANE_Status usb_request_sense(SnapScan_Scanner *pss);
 
@@ -106,8 +96,8 @@ static SANE_Status snapscani_usb_cmd(int fd, const void *src, size_t src_size,
     static const char me[] = "snapscani_usb_cmd";
     int status;
 
-    DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-         fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
+    DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+         fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
     while(bqhead) {
         status = atomic_usb_cmd(fd, bqhead->src, bqhead->src_size, NULL, NULL);
@@ -141,19 +131,19 @@ static SANE_Status atomic_usb_cmd(int fd, const void *src, size_t src_size,
     int status;
     sigset_t all,oldset;
 
-    DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-         fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
+    DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+         fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
     /* Prevent the calling process from being killed */
     sigfillset(&all);
     sigprocmask(SIG_BLOCK, &all, &oldset);
 
     /* Make sure we are alone */
-    semop(sem_id, &sem_wait, 1);
+    snapscani_mutex_lock(&snapscan_mutex);
 
     status = usb_cmd(fd,src,src_size,dst,dst_size);
 
-    semop(sem_id, &sem_signal, 1);
+    snapscani_mutex_unlock(&snapscan_mutex);
 
     /* Now it is ok to be killed */
     sigprocmask(SIG_SETMASK, &oldset, NULL);
@@ -169,27 +159,74 @@ static SANE_Status snapscani_usb_open(const char *dev, int *fdp,
 
     DBG (DL_CALL_TRACE, "%s(%s)\n", me, dev);
 
-    if((sem_id = semget( ftok(dev,0x12), 1, IPC_CREAT | 0660 )) == -1) {
+    if(!snapscani_mutex_open(&snapscan_mutex, dev)) {
         DBG (DL_MAJOR_ERROR, "%s: Can't get semaphore\n", me);
         return SANE_STATUS_INVAL;
     }
-    semop(sem_id, &sem_signal, 1);
     usb_sense_handler=sense_handler;
     usb_pss = pss;
+    urb_counters->read_urbs = 0;
+    urb_counters->write_urbs = 0;    
     return sanei_usb_open(dev, fdp);
 }
 
 
 static void snapscani_usb_close(int fd) {
     static const char me[] = "snapscani_usb_close";
-    static union semun dummy_semun_arg;
-
+    SANE_Word vendor_id, product_id;
+    
     DBG (DL_CALL_TRACE, "%s(%d)\n", me, fd);
-    semctl(sem_id, 0, IPC_RMID, dummy_semun_arg);
+    DBG (DL_DATA_TRACE,"1st read %ld write %ld\n", urb_counters->read_urbs, urb_counters->write_urbs);
+    
+    /* Check if URB counting is needed. If yes, ensure the number of sent and
+       received URBs is even.
+       Odd number of URBs only cause problems with libusb and certain 
+       scanner models. On other scanner models, sending additional commands
+       seems to cause problems (e.g. 1212u_2).
+       If sanei_usb_get_vendor_product returns an error there's probably no
+       libusb, so everything's fine.
+    */
+    if (sanei_usb_get_vendor_product(fd, &vendor_id, &product_id) == SANE_STATUS_GOOD)
+    {
+        /* Exclude 1212u_2 */
+        if (!((vendor_id == USB_VENDOR_AGFA) && (product_id == USB_PRODUCT_1212U2)))        
+        {
+            if ((urb_counters->read_urbs & 0x01) && (urb_counters->write_urbs & 0x01))
+            {
+                char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
+        
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
+            }
+            else if (urb_counters->read_urbs & 0x01)
+            {
+                size_t read_bytes;
+                char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
+                char cmd2[] = {INQUIRY, 0, 0, 0, 120, 0};
+                char data[120];
+        
+                read_bytes = 120;
+                snapscani_usb_cmd (fd, cmd2, sizeof (cmd2), data, &read_bytes);
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
+            }
+            else if (urb_counters->write_urbs & 0x01)
+            {
+                size_t read_bytes;
+                char cmd[] = {INQUIRY, 0, 0, 0, 120, 0};
+                char data[120];
+        
+                read_bytes = 120;
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), data, &read_bytes);
+            }
+            DBG (DL_DATA_TRACE,"2nd read %ld write %ld\n", urb_counters->read_urbs, 
+                urb_counters->write_urbs);
+        }
+    }
+    urb_counters->read_urbs = 0;
+    urb_counters->write_urbs = 0;
+    snapscani_mutex_close(&snapscan_mutex);
     sanei_usb_close(fd);
 }
 
-
 static int usb_cmdlen(int cmd)
 {
     switch(cmd) {
@@ -238,9 +275,11 @@ static SANE_Status usb_write(int fd, const void *buf, size_t n) {
 
     status = sanei_usb_write_bulk(fd, (const SANE_Byte*)buf, &bytes_written);
     if(bytes_written != n) {
-        DBG (DL_MAJOR_ERROR, "%s Only %d bytes written\n",me,bytes_written);
+      DBG (DL_MAJOR_ERROR, "%s Only %lu bytes written\n",me, (u_long) bytes_written);
         status = SANE_STATUS_IO_ERROR;
     }
+    urb_counters->write_urbs += (bytes_written + 7) / 8;
+    DBG (DL_DATA_TRACE, "Written %lu bytes\n", (u_long) bytes_written);
     return status;
 }
 
@@ -252,15 +291,17 @@ static SANE_Status usb_read(SANE_Int fd, void *buf, size_t n) {
 
     status = sanei_usb_read_bulk(fd, (SANE_Byte*)buf, &bytes_read);
     if (bytes_read != n) {
-        DBG (DL_MAJOR_ERROR, "%s Only %d bytes read\n",me,bytes_read);
+        DBG (DL_MAJOR_ERROR, "%s Only %lu bytes read\n",me, (u_long) bytes_read);
         status = SANE_STATUS_IO_ERROR;
     }
-
+    urb_counters->read_urbs += ((63 + bytes_read) / 64); 
     DBG(DL_DATA_TRACE, "%s: reading: %s\n",me,usb_debug_data(dbgmsg,buf,n));
+    DBG(DL_DATA_TRACE, "Read %lu bytes\n", (u_long) bytes_read);
     return status;
 }
 
-static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_status)
+static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_status,
+                                   char command)
 {
     static const char me[] = "usb_read_status";
     unsigned char status_buf[8];
@@ -282,7 +323,13 @@ static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_sta
         return SANE_STATUS_GOOD;
     case CHECK_CONDITION:
         if (usb_pss != NULL) {
-            return usb_request_sense(usb_pss);
+            if (command != REQUEST_SENSE) {
+                return usb_request_sense(usb_pss);
+            }
+            else {
+                /* Avoid recursive calls of usb_request_sense */
+                return SANE_STATUS_GOOD;
+            }
         } else {
             DBG (DL_MAJOR_ERROR, "%s: scanner structure not set, returning default error\n",
                 me);
@@ -303,15 +350,16 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   static const char me[] = "usb_cmd";
   int status,tstatus;
   int cmdlen,datalen;
+  char command;
 
-  DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-       fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
-
+  DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+       fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
   /* Since the  "Send Diagnostic" command isn't supported by
      all Snapscan USB-scanners it's disabled .
   */
-  if(((const char *)src)[0] == SEND_DIAGNOSTIC)
+  command = ((const char *)src)[0];
+  if(command == SEND_DIAGNOSTIC)
       return(SANE_STATUS_GOOD);
 
   cmdlen = usb_cmdlen(*((const char *)src));
@@ -323,7 +371,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   RETURN_ON_FAILURE( usb_write(fd,src,cmdlen) );
 
   /* Read status */
-  RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+  RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
 
   /* Send data only if the scanner is expecting it */
   if(datalen > 0 && (tstatus == TRANSACTION_WRITE)) {
@@ -331,7 +379,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
       RETURN_ON_FAILURE( usb_write(fd, ((const SANE_Byte *) src) + cmdlen, datalen) );
 
       /* Read status */
-      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
   }
 
   /* Receive data only when new data is waiting */
@@ -339,7 +387,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
       RETURN_ON_FAILURE( usb_read(fd,dst,*dst_size) );
 
       /* Read status */
-      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
   }
 
   if(tstatus != TRANSACTION_COMPLETED) {
@@ -377,7 +425,7 @@ static int enqueue_bq(int fd,const void *src, size_t src_size)
     static const char me[] = "enqueue_bq";
     struct usb_busy_queue *bqe;
 
-    DBG (DL_CALL_TRACE, "%s(%d,%p,%d)\n", me, fd,src,src_size);
+    DBG (DL_CALL_TRACE, "%s(%d,%p,%lu)\n", me, fd,src, (u_long) src_size);
 
     if((bqe = malloc(sizeof(struct usb_busy_queue))) == NULL)
         return -1;
@@ -454,8 +502,100 @@ static SANE_Status usb_request_sense(SnapScan_Scanner *pss) {
     return status;
 }
 
+#if defined USE_PTHREAD || defined HAVE_OS2_H || defined __BEOS__
+static SANE_Status snapscani_usb_shm_init(void)
+{
+    unsigned int shm_size = sizeof(struct urb_counters_t);
+    urb_counters = (struct urb_counters_t*) malloc(shm_size);
+    if (urb_counters == NULL)
+    {
+        return SANE_STATUS_NO_MEM;
+    }
+    memset(urb_counters, 0, shm_size);
+    return SANE_STATUS_GOOD;
+        
+}
+
+static void snapscani_usb_shm_exit(void)
+{
+    if (urb_counters)
+    {
+        free ((void*)urb_counters);
+        urb_counters = NULL;
+    }
+}
+#else
+#include <sys/ipc.h>
+#include <sys/shm.h>
+static SANE_Status snapscani_usb_shm_init(void)
+{
+    unsigned int shm_size = sizeof(struct urb_counters_t);
+    void* shm_area = NULL;
+    int shm_id = shmget (IPC_PRIVATE, shm_size, IPC_CREAT | SHM_R | SHM_W);
+    if (shm_id == -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot create shared memory segment: %s\n",
+            strerror (errno));
+        return SANE_STATUS_NO_MEM;
+    }
+    
+    shm_area = shmat (shm_id, NULL, 0);
+    if (shm_area == (void *) -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot attach to shared memory segment: %s\n",
+            strerror (errno));
+        shmctl (shm_id, IPC_RMID, NULL);
+        return SANE_STATUS_NO_MEM;
+    }
+    
+    if (shmctl (shm_id, IPC_RMID, NULL) == -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot remove shared memory segment id: %s\n",
+            strerror (errno));
+        shmdt (shm_area);
+        shmctl (shm_id, IPC_RMID, NULL);
+        return SANE_STATUS_NO_MEM;
+    }
+    urb_counters = (struct urb_counters_t*) shm_area;
+    memset(urb_counters, 0, shm_size);
+    return SANE_STATUS_GOOD;
+}
+
+static void snapscani_usb_shm_exit(void)
+{
+    if (urb_counters)
+    {
+        shmdt (urb_counters);
+        urb_counters = NULL;
+    }
+}
+#endif
 /*
  * $Log: snapscan-usb.c,v $
+ * Revision 1.21  2005/11/02 19:22:06  oliver-guest
+ * Fixes for Benq 5000
+ *
+ * Revision 1.20  2005/07/18 17:37:37  oliver-guest
+ * ZETA changes for SnapScan backend
+ *
+ * Revision 1.19  2004/10/03 17:34:36  hmg-guest
+ * 64 bit platform fixes (bug #300799).
+ *
+ * Revision 1.18  2004/06/16 19:52:26  oliver-guest
+ * Don't enforce even number of URB packages on 1212u_2. Fixes bug #300753.
+ *
+ * Revision 1.17  2004/06/06 14:50:36  oliver-guest
+ * Use shared memory functions only when needed
+ *
+ * Revision 1.16  2004/05/26 22:37:01  oliver-guest
+ * Use shared memory for urb counters in snapscan backend
+ *
+ * Revision 1.15  2004/04/09 11:59:02  oliver-guest
+ * Fixes for pthread implementation
+ *
+ * Revision 1.14  2004/04/08 22:48:13  oliver-guest
+ * Use URB counting in snapscan-usb.c (thanks to Jose Alberto Reguero)
+ *
  * Revision 1.13  2003/11/08 09:50:27  oliver-guest
  * Fix TPO scanning range for Epson 1670
  *
