@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002-2003 Frank Zago (fzago at austin dot rr dot com)
-   Copyright (C) 2003 Gerard Klaver
+   Copyright (C) 2002-2003 Frank Zago (sane at zago dot net)
+   Copyright (C) 2003-2005 Gerard Klaver (gerard at gkall dot hobby dot nl)
 
    This file is part of the SANE package.
    
@@ -43,15 +43,29 @@
 */
 
 /*
-   $Id: teco2.c,v 1.6 2003/04/25 23:43:45 fzago Exp $
-   TECO scanner VM3575, VM6565, VM6575, VM6586, VM356A
-   Patch for VM356A Gerard Klaver v  2003/02/14
-   update 2003/03/19, traces, tests, Gerard Klaver, Michael Hoeller
+   $Id: teco2.c,v 1.17 2005/09/19 19:57:48 fzago-guest Exp $
+   TECO scanner VM3575, VM656A, VM6575, VM6586, VM356A, VM3564
+   update 2003/02/14, Patch for VM356A Gerard Klaver
+   update 2003/03/19, traces, tests VM356A Gerard Klaver, Michael Hoeller
+   update 2003/07/19, white level calibration, color modes VM3564, VM356A, VM3575
+                      Gerard Klaver, Michael Hoeller
+   update 2004/01/15, white level , red, green, and blue calibration and
+		      leave out highest and lowest value and then divide
+                      VM3564, VM356A and VM3575: Gerard Klaver
+   update 2004/08/04, white level, red, green and blue calibration for the VM6575
+                      changed default SANE_TECO_CAL_ALGO to 1 for VM3564 and VM6575
+		      preview value changed to 75 dpi for VM6575
+		      leave out highest and lowest value and then divide
+		      VM656A, VM6586
+   update 2004/08/05, use of SANE_VALUE_SCAN_MODE_LINEART, _GRAY, and _COLOR,
+                      changed use of %d to %ld (when bytes values are displayed)
+   update 2005/03/04, use of __sane_unused__
+   update 2005/07/29. Removed using teco_request_sense (dev) routine for VM3564
 */
 
 /*--------------------------------------------------------------------------*/
 
-#define BUILD 5			/* 2003/04/18 */
+#define BUILD 9			/* 2005/07/29 */
 #define BACKEND_NAME teco2
 #define TECO2_CONFIG_FILE "teco2.conf"
 
@@ -91,16 +105,16 @@
 #endif
 
 /* For debugging purposes: output a stream straight out from the
- * scanner without reordering the colors. */
+ * scanner without reordering the colors, 0=normal, 1 = raw. */
 static int raw_output = 0;
 
 /*--------------------------------------------------------------------------*/
 
 /* Lists of possible scan modes. */
 static SANE_String_Const scan_mode_list[] = {
-  BLACK_WHITE_STR,
-  GRAYSCALE_STR,
-  COLOR_STR,
+	SANE_VALUE_SCAN_MODE_LINEART,
+	SANE_VALUE_SCAN_MODE_GRAY,
+	SANE_VALUE_SCAN_MODE_COLOR,
   NULL
 };
 
@@ -156,6 +170,30 @@ static const SANE_Range threshold_range = {
 
 /*--------------------------------------------------------------------------*/
 
+static const SANE_Range red_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
+static const SANE_Range green_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
+static const SANE_Range blue_level_range = {
+  0,				/* minimum */
+  64,				/* maximum */
+  1				/* quantization */
+};
+
+/*--------------------------------------------------------------------------*/
+
 /* Gamma range */
 static const SANE_Range gamma_range = {
   0,				/* minimum */
@@ -167,22 +205,17 @@ static const SANE_Range gamma_range = {
 
 static const struct dpi_color_adjust vm3564_dpi_color_adjust[] = {
 
-  /*dpi, x, y, z, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
+  /*dpi, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
   {25, 1, 0, 2, 0, 0},
-  {40, 1, 0, 2, 0, 0},
-  {50, 1, 0, 2, 0, 1},
-  {60, 1, 0, 2, 1, 0},
-  {72, 1, 2, 0, 0, 1},
+  {50, 1, 2, 0, 0, 1},
   {75, 1, 0, 2, 1, 1},
-  {100, 2, 1, 0, 0, 1},
-  {120, 1, 0, 2, 1, 1},
   {150, 1, 0, 2, 1, 2},
-  {160, 1, 0, 2, 1, 1},
-  {175, 1, 0, 2, 1, 1},
-  {180, 1, 0, 2, 1, 1},
-  {200, 2, 1, 0, 0, 1},
-  {225, 1, 0, 2, 1, 2},
-  {300, 1, 0, 2, 1, 2},
+  {225, 1, 0, 2, 1, 3},
+  {300, 1, 0, 2, 1, 4},
+  {375, 1, 0, 2, 1, 5},
+  {450, 1, 0, 2, 1, 6},
+  {525, 1, 0, 2, 1, 7},
+  {600, 1, 0, 2, 1, 8},
   /* must be the last entry */
   {0, 0, 0, 0, 0, 0}
 };
@@ -190,50 +223,55 @@ static const struct dpi_color_adjust vm3564_dpi_color_adjust[] = {
 /* Used for VM356A only */
 
 static const struct dpi_color_adjust vm356a_dpi_color_adjust[] = {
-  /*dpi, x, y, z, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
-  {25, 1, 0, 2, 0, 0},		/* ok */
-  {50, 1, 0, 2, 0, 1},
-  {75, 1, 0, 2, 1, 1},		/* ok */
-  {150, 1, 0, 2, 1, 2},		/* ok */
-  {160, 1, 0, 2, 1, 2},		/* ok */
-  {225, 1, 0, 2, 1, 2},		/* ok */
-  {300, 1, 0, 2, 1, 2},		/* ok */
 
-  {305, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {310, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {315, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {375, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {380, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {385, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {450, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {455, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {525, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
-  {600, 1, 0, 2, 1, 2},		/* ok, but wrong proportion */
+  /* All values with ydpi > 300 result in a wrong proportion for       */
+  /* the scan. The proportion can be adjusted with the following       */
+  /* command: convert -geometry (dpi/max_xdpi * 100%)x100%             */
+  /* max_xdpi is for the vm356a constant with 300 dpi                  */
+  /* e.g. 600dpi adjust with: convert -geometry 200%x100%              */
 
-/* text Michael Hoeller */
-  /* all increments by 5 from 20 to 600 are checked  */
-/* with 85, 165, 155 good picture but not colur setup works */
-/* with 65, 200 and 320 crashs the driver    */
+  /* All resolutions in increments of 25 are testede, only the shown   */
+  /* bring back good results                                           */
 
-/*  {0, 1, 0, 2, 1, 1} general setup NOK for all dpi values */
-/*   150, 225 and 300 should be OK*/
+  /*dpi, color sequence R G or B, 0 (-) or 1 (+) color skewing, lines skewing */
+  {25, 1, 0, 2, 0, 0},
+  {50, 1, 2, 0, 0, 1},
+  {75, 1, 0, 2, 1, 1},
+  {150, 1, 0, 2, 1, 2},
+  {225, 1, 0, 2, 1, 3},
+  {300, 1, 0, 2, 1, 4},
+  {375, 1, 0, 2, 1, 5},
+  {450, 1, 0, 2, 1, 6},
+  {525, 1, 0, 2, 1, 7},
+  {600, 1, 0, 2, 1, 8},
 
   /* must be the last entry */
   {0, 0, 0, 0, 0, 0}
 };
+
 /* Used for the VM3575 only */
 
 static const struct dpi_color_adjust vm3575_dpi_color_adjust[] = {
 
+  /* All values with ydpi > 300 result in a wrong proportion for */
+  /* the scan. The proportion can be adjusted with the following */
+  /* command: convert -geometry (dpi/max_xdpi * 100%)x100%       */
+  /* max_xdpi is for the vm3575 constant with 300 dpi            */
+  /* e.g. 600dpi adjust with: convert -geometry 200%x100%        */
+
   {50, 2, 0, 1, 1, 1},
   {60, 2, 1, 0, 0, 2},
-  {75, 2, 0, 1, 1, 2},
+/* {75, 2, 0, 1, 1, 2},           NOK, effects in scan, also with twain driver*/
   {100, 2, 1, 0, 0, 3},
   {120, 2, 0, 1, 1, 3},
   {150, 2, 0, 1, 1, 4},
-  {180, 2, 1, 0, 0, 4},
-  {225, 2, 0, 1, 0, 6},
+/* {180, 2, 1, 0, 0, 4},          NOK, skewing problem*/
+  {225, 2, 0, 1, 1, 6},
   {300, 2, 0, 1, 1, 8},
+  {375, 2, 0, 1, 1, 10},
+  {450, 2, 0, 1, 1, 12},
+  {525, 2, 0, 1, 1, 14},
+  {600, 2, 0, 1, 1, 16},
 
   /* must be the last entry */
   {0, 0, 0, 0, 0, 0}
@@ -282,7 +320,7 @@ static const struct dpi_color_adjust default_dpi_color_adjust[1] = {
 };
 
 /* For all scanners. Must be reasonable (eg. between 50 and 300) and
- * appear in the vm3575_dpi_color_adjust list. */
+ * appear in the ...._dpi_color_adjust list of all supported scanners. */
 #define DEF_RESOLUTION 150
 
 /*--------------------------------------------------------------------------*/
@@ -295,28 +333,53 @@ static const struct scanners_supported scanners[] = {
    "Relisys", "AVEC II S3",
    {1, 600, 1},			/* resolution */
    300, 600,			/* max x and Y resolution */
-   2550, 12, 3, 0,			/* calibration */
+   2550, 12, 3, 1,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm3564_dpi_color_adjust},
+
   {6, "TECO VM356A",
    TECO_VM356A,
    "Relisys", "APOLLO Express 3",
    {1, 600, 1},			/* resolution */
    300, 600,			/* max x and Y resolution */
-   2550, 12, 3, 0,			/* calibration */
+   2550, 12, 3, 1,		/* calibration */
+   /* dots/inch * x-length, calibration samples, tot.bytes for 3 colors, default SANE_TECO2_CAL_ALGO value */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm356a_dpi_color_adjust},
+
+  {6, "TECO VM356A",
+   TECO_VM356A,
+   "Primax", "Jewel 4800",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 3, 1,		/* calibration */
+   /* dots/inch * x-length, calibration samples, tot.bytes for 3 colors, default SANE_TECO2_CAL_ALGO value */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm356a_dpi_color_adjust},
+
   {6, "TECO VM3575",
    TECO_VM3575,
    "Relisys", "SCORPIO Super 3",
    {1, 600, 1},			/* resolution */
    300, 600,			/* max x and Y resolution */
-   2550, 12, 6, 1,			/* calibration */
+   2550, 12, 6, 1,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm3575_dpi_color_adjust},
+
+  {6, "TECO VM3575",
+   TECO_VM3575,
+   "Relisys", "AVEC Super 3",
+   {1, 600, 1},			/* resolution */
+   300, 600,			/* max x and Y resolution */
+   2550, 12, 6, 1,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm3575_dpi_color_adjust},
+
   {6, "TECO VM656A",
    TECO_VM656A,
    "Relisys", "APOLLO Express 6",
@@ -326,15 +389,27 @@ static const struct scanners_supported scanners[] = {
    {SANE_FIX (0), SANE_FIX (210), 0},
    {SANE_FIX (0), SANE_FIX (297), 0},
    vm656a_dpi_color_adjust},
+
   {6, "TECO VM6575",
    TECO_VM6575,
    "Relisys", "SCORPIO Pro",
    {1, 600, 1},			/* resolution */
    600, 1200,			/* max x and Y resolution */
-   5100, 8, 6, 0,			/* calibration */
+   5100, 8, 6, 1,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm6575_dpi_color_adjust},
+
+  {6, "TECO VM6575",
+   TECO_VM6575,
+   "Primax", "Profi 9600",
+   {1, 600, 1},			/* resolution */
+   600, 1200,			/* max x and Y resolution */
+   5100, 8, 6, 1,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm6575_dpi_color_adjust},
+
   {6, "TECO VM6586",
    TECO_VM6586,
    "Relisys", "SCORPIO Pro-S",
@@ -343,6 +418,16 @@ static const struct scanners_supported scanners[] = {
    5100, 8, 6, 0,		/* calibration */
    {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
    {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
+   vm6586_dpi_color_adjust},
+
+  {6, "TECO VM6586",
+   TECO_VM6586,
+   "Primax", "Profi 19200",
+   {1, 600, 1},			/* resolution */
+   600, 1200,			/* max x and Y resolution */
+   5100, 8, 6, 0,		/* calibration */
+   {SANE_FIX (0), SANE_FIX (8.5 * MM_PER_INCH), 0},
+   {SANE_FIX (0), SANE_FIX (11.7 * MM_PER_INCH), 0},
    vm6586_dpi_color_adjust}
 };
 
@@ -680,16 +765,36 @@ teco_identify_scanner (Teco_Scanner * dev)
    * backend supports that one. */
   for (i = 0; i < NELEMS (scanners); i++)
     {
-
       if (dev->scsi_type == scanners[i].scsi_type &&
 	  strcmp (dev->scsi_teco_name, scanners[i].scsi_teco_name) == 0)
 	{
 
 	  DBG (DBG_error, "teco_identify_scanner: scanner supported\n");
 
-	  dev->def = &(scanners[i]);
+	  /*patch for VM356A Jewel or Apollo text in frontend-+ others-------------- */
+
+	  if (strncmp (dev->scsi_vendor, "RELISYS", 1) == 0 ||
+	      strncmp (dev->scsi_vendor, "       ", 1) == 0)
+	    {
+	      DBG (DBG_error,
+		   "teco_identify_scanner: scanner detected with this teco_name and first brand/name entry in table\n");
+
+	      dev->def = &(scanners[i]);
+
+	      return (SANE_TRUE);
+	    }
+	  else
+	    {
+	      i++;
+
+	      DBG (DBG_error,
+		   "teco_identify_scanner: scanner detected with this teco_name and second brand/name entry in table\n");
+
+	      dev->def = &(scanners[i]);
+
+	      return (SANE_TRUE);
+	    }
 
-	  return (SANE_TRUE);
 	}
     }
 
@@ -1019,10 +1124,11 @@ teco_wait_for_data (Teco_Scanner * dev)
   return SANE_STATUS_DEVICE_BUSY;
 }
 
-/* Do the calibration stuff. Get 12 lines of data. Each pixel is coded
- * in 6 bytes (2 per color). To do the calibration, allocates an array
- * big enough for one line, read the 12 lines of calibration, and do a
- * simple average. The input line is 1 raster for each color. However
+/* Do the calibration stuff. Get 12 or 8 lines of data. Each pixel is coded
+ * in 6 bytes (2 per color) or 3 bytes (3564 and 356A). To do the calibration,
+ * allocates an array big enough for one line, read the 12 or 8 lines of calibration,
+ * subtract the highest and lowest value and do a average. 
+ * The input line is 1 raster for each color. However
  * the output line is interlaced (ie RBG for the first pixel, then RGB
  * for the second, and so on...). The output line are the value to use
  * to compensate for the white point. 
@@ -1048,46 +1154,106 @@ teco_do_calibration (Teco_Scanner * dev)
   size_t size;
   int i;
   int j;
-  int colsub;
-  int *tmp_buf;			/* hold the temporary calibration */
-  size_t tmp_buf_size;
+  int colsub0_0, colsub0_1, colsub0_2;
+  int colsub1_0, colsub1_1, colsub1_2;
+  int *tmp_buf, *tmp_min_buf, *tmp_max_buf;			/* hold the temporary calibration */
+  size_t tmp_buf_size, tmp_min_buf_size, tmp_max_buf_size;
   const char *calibration_algo;
   int cal_algo;
 
+  colsub0_0 = 0;
+  colsub0_1 = 0;
+  colsub0_2 = 0;
+  colsub1_0 = 0;
+  colsub1_1 = 0;
+  colsub1_2 = 0;
+	
   DBG (DBG_proc, "teco_do_calibration: enter\n");
 
   /* Get default calibration algorithm. */
   cal_algo = dev->def->cal_algo;
-  if ((calibration_algo = getenv("SANE_TECO2_CAL_ALGO")) != NULL) {
-	  cal_algo = atoi(calibration_algo);
-  }
-  if (cal_algo != 0 && cal_algo != 1) {
-	  DBG(DBG_error, "Invalid calibration algorithm (%d)\n", cal_algo);
-	  cal_algo = 0;
-  }
+  if ((calibration_algo = getenv ("SANE_TECO2_CAL_ALGO")) != NULL)
+    {
+      cal_algo = atoi (calibration_algo);
+    }
+  if (cal_algo != 0 && cal_algo != 1)
+    {
+      DBG (DBG_error, "Invalid calibration algorithm (%d)\n", cal_algo);
+      cal_algo = 0;
+    }
 
   switch (dev->def->tecoref)
     {
     case TECO_VM3564:
     case TECO_VM356A:
-      colsub = 0x110;
+      /* white level red, green and blue calibration correction */
+      /* 0x110 or 272 is middle value */
+      colsub0_1 = 240 + (dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub0_2 = 240 + (dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub0_0 = 240 + (dev->val[OPT_WHITE_LEVEL_B].w);
+      /* 14000 is middle value */
+      colsub1_1 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub1_2 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub1_0 = 12720 + (40 * dev->val[OPT_WHITE_LEVEL_B].w);
       break;
     case TECO_VM3575:
-      colsub = 0x1100;
+    case TECO_VM6575:
+      /* 0x1100 or 4352 is middle value */
+      colsub0_1 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub0_2 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub0_0 = 4096 + (8 * dev->val[OPT_WHITE_LEVEL_B].w);
+      /* 4206639 is middle value */
+      colsub1_1 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_R].w);
+      colsub1_2 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_G].w);
+      colsub1_0 = 4078639 + (4000 * dev->val[OPT_WHITE_LEVEL_B].w);
+      break;
+      /* old default value */
+    case TECO_VM656A:
+    case TECO_VM6586:
+      colsub0_1 = 0x1000;
+      colsub0_2 = 0x1000;
+      colsub0_0 = 0x1000;
+      colsub1_1 = 4206639;
+      colsub1_2 = 4206639;
+      colsub1_0 = 4206639;
       break;
-      /* For VM6575, 656A, 6586 until otherwise default value is used */
     default:
-      colsub = 0x1000;
+      colsub0_1 = 0x1000;
+      colsub0_2 = 0x1000;
+      colsub0_0 = 0x1000;
+      colsub1_1 = 4206639;
+      colsub1_2 = 4206639;
+      colsub1_0 = 4206639;
       break;
     }
 
   tmp_buf_size = dev->def->cal_length * 3 * sizeof (int);
+  tmp_min_buf_size = dev->def->cal_length * 3 * sizeof (int);
+  tmp_max_buf_size = dev->def->cal_length * 3 * sizeof (int);
   tmp_buf = malloc (tmp_buf_size);
+  tmp_min_buf = malloc (tmp_min_buf_size);
+  tmp_max_buf = malloc (tmp_max_buf_size);
   memset (tmp_buf, 0, tmp_buf_size);
-  if (tmp_buf == NULL)
+  switch (dev->def->tecoref)
     {
-      DBG (DBG_proc, "teco_do_calibration: not enough memory (%d bytes)\n",
-	   tmp_buf_size);
+    case TECO_VM3564:
+    case TECO_VM356A:
+  	memset (tmp_min_buf, 0xff, tmp_min_buf_size);
+  	memset (tmp_max_buf, 0x00, tmp_max_buf_size);
+      break;
+    case TECO_VM3575:
+    case TECO_VM656A:
+    case TECO_VM6575:
+    case TECO_VM6586:
+  	memset (tmp_min_buf, 0xffff, tmp_min_buf_size);
+  	memset (tmp_max_buf, 0x0000, tmp_max_buf_size);
+      break;
+    }
+  
+  if ((tmp_buf == NULL) || (tmp_min_buf == NULL) || (tmp_max_buf == NULL))
+    {
+      DBG (DBG_proc, "teco_do_calibration: not enough memory (%ld bytes)\n",
+	   (long) tmp_buf_size);
       return (SANE_STATUS_NO_MEM);
     }
 
@@ -1150,34 +1316,142 @@ teco_do_calibration (Teco_Scanner * dev)
 	    case TECO_VM6586:
 	      tmp_buf[3 * j + 0] +=
 		(dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 0]  >> ((dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0]))
+	      {
+		      tmp_min_buf[3 * j + 0] = (dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 0]  < ((dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0]))
+	      {
+		      tmp_max_buf[3 * j + 0] = (dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      }
 	      tmp_buf[3 * j + 1] +=
 		(dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 1]  >> ((dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2]))
+	      {
+		      tmp_min_buf[3 * j + 1] = (dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 1]  < ((dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2]))
+	      {
+		      tmp_max_buf[3 * j + 1] = (dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      }
 	      tmp_buf[3 * j + 2] +=
 		(dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 2]  >> ((dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4]))
+	      {
+		      tmp_min_buf[3 * j + 2] = (dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 2]  < ((dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4]))
+	      {
+		      tmp_max_buf[3 * j + 2] = (dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      }
 	      break;
 	    case TECO_VM3564:
 	    case TECO_VM356A:
 	      tmp_buf[3 * j + 0] += dev->buffer[3 * j + 0];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 0]  >> dev->buffer[3 * j + 0])
+	      {
+		      tmp_min_buf[3 * j + 0] = dev->buffer[3 * j + 0];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 0]  < dev->buffer[3 * j + 0]) 
+	      {
+		      tmp_max_buf[3 * j + 0] = dev->buffer[3 * j + 0];
+	      }
 	      tmp_buf[3 * j + 1] += dev->buffer[3 * j + 1];
+	      /* get lowest value */
+	      if (tmp_min_buf[3 * j + 1]  >> dev->buffer[3 * j + 1]) 
+	      {
+		      tmp_min_buf[3 * j + 1]  = dev->buffer[3 * j + 1];
+	      }
+	      /* get hightest value */
+	      if (tmp_max_buf[3 * j + 1]  < dev->buffer[3 * j + 1]) 
+	      {
+		      tmp_max_buf[3 * j + 1]  = dev->buffer[3 * j + 1];
+	      }
 	      tmp_buf[3 * j + 2] += dev->buffer[3 * j + 2];
-	      break;
+              /* get lowest value */
+	      if (tmp_min_buf[3 * j + 2]  >> dev->buffer[3 * j + 2]) 
+	      {
+		      tmp_min_buf[3 * j + 2]  = dev->buffer[3 * j + 2];
+	      }
+	      /* get highest value */
+	      if (tmp_max_buf[3 * j + 2]  < dev->buffer[3 * j + 2]) 
+	      {
+		      tmp_max_buf[3 * j + 2]  = dev->buffer[3 * j + 2];
+	      }
+		break;
+/*	    default:
+	      tmp_buf[3 * j + 0] +=
+		(dev->buffer[6 * j + 1] << 8) + dev->buffer[6 * j + 0];
+	      tmp_buf[3 * j + 1] +=
+		(dev->buffer[6 * j + 3] << 8) + dev->buffer[6 * j + 2];
+	      tmp_buf[3 * j + 2] +=
+		(dev->buffer[6 * j + 5] << 8) + dev->buffer[6 * j + 4];
+	      break;  */
 	    }
 	}
     }
 
-  /*hexdump (DBG_info2, "calibration before average:", tmp_buf, tmp_buf_size); */
-
-  /* Do the average. Since we got 12 lines, divide all values by 12
+  /* hexdump (DBG_info2, "calibration before average:", tmp_buf, tmp_buf_size); */
+  /* hexdump (DBG_info2, "calibration before average min value:", tmp_min_buf, tmp_min_buf_size); */
+  /* hexdump (DBG_info2, "calibration before average max value:", tmp_max_buf, tmp_max_buf_size); */
+  
+  /* Do the average. Since we got 12 or 8 lines, divide all values by 10 or 6
    * and create the final calibration value that compensates for the
    * white values read. */
-  for (j = 0; j < (3 * dev->def->cal_length); j++)
-    {
-
-		if (cal_algo == 1) {
-			tmp_buf[j] = ( 4206639 * dev->def->cal_lines )/tmp_buf[j];
-		} else {
-			tmp_buf[j] = colsub - (tmp_buf[j] / dev->def->cal_lines);
+  switch (dev->def->tecoref)
+  {
+	case TECO_VM356A:
+	case TECO_VM3564:
+	case TECO_VM3575:
+   	case TECO_VM6575:
+	case TECO_VM656A:
+	case TECO_VM6586:
+  		for (j = 0; j < dev->def->cal_length; j++)
+		{
+	       /* subtract lowest and highest value */
+			tmp_buf[j] = tmp_buf[j] - (tmp_min_buf[j] + tmp_max_buf[j]); 
+			tmp_buf[j + dev->def->cal_length] = tmp_buf[j + dev->def->cal_length] 
+				- (tmp_min_buf[j + dev->def->cal_length] 
+				+ tmp_max_buf[j + dev->def->cal_length]); 
+			tmp_buf[j + 2 * dev->def->cal_length] = tmp_buf[j + 2 * dev->def->cal_length] 
+				- (tmp_min_buf[j + 2 * dev->def->cal_length] 
+				+ tmp_max_buf[j + 2 *dev->def->cal_length]); 
+		/* sequence colors first color row one then two and last three   */
+      			if (cal_algo == 1) 
+			{
+	  		tmp_buf[j] = (colsub1_0 * (dev->def->cal_lines - 2)) / tmp_buf[j];
+	  		tmp_buf[j + dev->def->cal_length] = (colsub1_1 * (dev->def->cal_lines - 2)) 
+				/ tmp_buf[j + dev->def->cal_length];
+	  		tmp_buf[j + 2 * dev->def->cal_length] = (colsub1_2 * (dev->def->cal_lines - 2)) 
+				/ tmp_buf[j + 2 * dev->def->cal_length];
+			}
+      			else 
+			{
+	  		tmp_buf[j] = colsub0_0 - (tmp_buf[j] / (dev->def->cal_lines - 2));
+	  		tmp_buf[j + dev->def->cal_length] = colsub0_1 - (tmp_buf[j + dev->def->cal_length] 
+								/ (dev->def->cal_lines - 2));
+	  		tmp_buf[j + 2 * dev->def->cal_length] = colsub0_2 
+				- (tmp_buf[j + 2 * dev->def->cal_length] / (dev->def->cal_lines - 2));
+			}
 		}
+	      	break;
+/*		default:
+  		for (j = 0; j < (3 * dev->def->cal_length); j++)
+		{
+      			if (cal_algo == 1)
+	  		tmp_buf[j] = (colsub1_1 * dev->def->cal_lines) / tmp_buf[j];
+      			else 
+	  		tmp_buf[j] = colsub0_1 - (tmp_buf[j] / dev->def->cal_lines);
+		}
+	      	break;   */
     }
 
   /*hexdump (DBG_info2, "calibration after average:", tmp_buf, tmp_buf_size); */
@@ -1395,15 +1669,13 @@ teco_scan (Teco_Scanner * dev)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-teco_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+teco_sense_handler (int scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
 
   DBG (DBG_proc, "teco_sense_handler (scsi_fd = %d)\n", scsi_fd);
 
-  arg = arg;			/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1781,6 +2053,36 @@ teco_init_options (Teco_Scanner * dev)
   dev->opt[OPT_PREVIEW].cap = SANE_CAP_SOFT_DETECT | SANE_CAP_SOFT_SELECT;
   dev->val[OPT_PREVIEW].w = SANE_FALSE;
 
+  /* red level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_R].name = SANE_NAME_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].title = SANE_TITLE_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].desc = SANE_DESC_WHITE_LEVEL_R;
+  dev->opt[OPT_WHITE_LEVEL_R].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_R].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_R].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_R].constraint.range = &red_level_range;
+  dev->val[OPT_WHITE_LEVEL_R].w = 32;	/* to get middle value */
+
+  /* green level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_G].name = SANE_NAME_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].title = SANE_TITLE_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].desc = SANE_DESC_WHITE_LEVEL_G;
+  dev->opt[OPT_WHITE_LEVEL_G].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_G].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_G].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_G].constraint.range = &green_level_range;
+  dev->val[OPT_WHITE_LEVEL_G].w = 32;	/* to get middle value */
+  
+  /* blue level calibration manual correction */
+  dev->opt[OPT_WHITE_LEVEL_B].name = SANE_NAME_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].title = SANE_TITLE_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].desc = SANE_DESC_WHITE_LEVEL_B;
+  dev->opt[OPT_WHITE_LEVEL_B].type = SANE_TYPE_INT;
+  dev->opt[OPT_WHITE_LEVEL_B].unit = SANE_UNIT_NONE;
+  dev->opt[OPT_WHITE_LEVEL_B].constraint_type = SANE_CONSTRAINT_RANGE;
+  dev->opt[OPT_WHITE_LEVEL_B].constraint.range = &blue_level_range;
+  dev->val[OPT_WHITE_LEVEL_B].w = 32;	/* to get middle value */
+  
   /* Lastly, set the default scan mode. This might change some
    * values previously set here. */
   sane_control_option (dev, OPT_MODE, SANE_ACTION_SET_VALUE,
@@ -2114,8 +2416,8 @@ teco_fill_image (Teco_Scanner * dev)
 	  return status;
 	}
 
-      DBG (DBG_info, "teco_fill_image: real bytes left = %d\n",
-	   dev->real_bytes_left);
+      DBG (DBG_info, "teco_fill_image: real bytes left = %ld\n",
+	  (long) dev->real_bytes_left);
 
       if (dev->scan_mode == TECO_COLOR &&
 	  dev->def->tecoref != TECO_VM656A && raw_output == 0)
@@ -2202,7 +2504,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -2212,11 +2514,9 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-teco2 version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
-  DBG (DBG_error, "(C) 2002 by Frank Zago\n");
+  DBG (DBG_error, "(C) 2002 - 2003 by Frank Zago, update 2003 - 2005 by Gerard Klaver\n");
 
   if (version_code)
     {
@@ -2251,15 +2551,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2405,6 +2703,9 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_CUSTOM_GAMMA:
 	case OPT_PREVIEW:
 	case OPT_THRESHOLD:
+	case OPT_WHITE_LEVEL_R:
+	case OPT_WHITE_LEVEL_G:
+	case OPT_WHITE_LEVEL_B:
 	  *(SANE_Word *) val = dev->val[option].w;
 	  return SANE_STATUS_GOOD;
 
@@ -2453,6 +2754,9 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_BR_X:
 	case OPT_THRESHOLD:
 	case OPT_RESOLUTION:
+	case OPT_WHITE_LEVEL_R:
+	case OPT_WHITE_LEVEL_G:
+	case OPT_WHITE_LEVEL_B:
 	  if (info)
 	    {
 	      *info |= SANE_INFO_RELOAD_PARAMS;
@@ -2492,13 +2796,17 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  dev->opt[OPT_DITHER].cap |= SANE_CAP_INACTIVE;
 	  dev->opt[OPT_FILTER_COLOR].cap |= SANE_CAP_INACTIVE;
 	  dev->opt[OPT_THRESHOLD].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_R].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_G].cap |= SANE_CAP_INACTIVE;
+	  dev->opt[OPT_WHITE_LEVEL_B].cap |= SANE_CAP_INACTIVE;
+
 
 	  /* This the default resolution range, except for the
 	   * VM3575, VM3564, VM356A and VM6586 in color mode. */
 	  dev->opt[OPT_RESOLUTION].constraint_type = SANE_CONSTRAINT_RANGE;
 	  dev->opt[OPT_RESOLUTION].constraint.range = &dev->def->res_range;
 
-	  if (strcmp (dev->val[OPT_MODE].s, BLACK_WHITE_STR) == 0)
+	  if (strcmp (dev->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_LINEART) == 0)
 	    {
 	      dev->scan_mode = TECO_BW;
 	      dev->depth = 8;
@@ -2506,29 +2814,80 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	      dev->opt[OPT_FILTER_COLOR].cap &= ~SANE_CAP_INACTIVE;
 	      dev->opt[OPT_THRESHOLD].cap &= ~SANE_CAP_INACTIVE;
 	    }
-	  else if (strcmp (dev->val[OPT_MODE].s, GRAYSCALE_STR) == 0)
+	  else if (strcmp (dev->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_GRAY) == 0)
 	    {
 	      dev->scan_mode = TECO_GRAYSCALE;
 	      dev->depth = 8;
-	      dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
-	      if (dev->val[OPT_CUSTOM_GAMMA].w)
+
+	      switch (dev->def->tecoref)
 		{
-		  dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &= ~SANE_CAP_INACTIVE;
+		case TECO_VM3564:
+		case TECO_VM356A:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  break;
+		case TECO_VM3575:
+		case TECO_VM6575:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &=
+			~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		case TECO_VM656A:
+		case TECO_VM6586:
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_GRAY].cap &=
+			~SANE_CAP_INACTIVE;
+		    }
+		  break;
 		}
 	      dev->opt[OPT_FILTER_COLOR].cap &= ~SANE_CAP_INACTIVE;
 	    }
-	  else if (strcmp (dev->val[OPT_MODE].s, COLOR_STR) == 0)
+	  else if (strcmp (dev->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_COLOR) == 0)
 	    {
 	      dev->scan_mode = TECO_COLOR;
 	      dev->depth = 8;
-	      dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
-	      if (dev->val[OPT_CUSTOM_GAMMA].w)
-		{
-		  dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
-		  dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
-		  dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
-		}
 
+	      switch (dev->def->tecoref)
+		{
+		case TECO_VM3564:
+		case TECO_VM356A:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  break;
+		case TECO_VM3575:
+		case TECO_VM6575:
+		  dev->opt[OPT_WHITE_LEVEL_R].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_G].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_WHITE_LEVEL_B].cap &= ~SANE_CAP_INACTIVE;
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		case TECO_VM656A:
+		case TECO_VM6586:
+		  dev->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
+		  if (dev->val[OPT_CUSTOM_GAMMA].w)
+		    {
+		      dev->opt[OPT_GAMMA_VECTOR_R].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_G].cap &= ~SANE_CAP_INACTIVE;
+		      dev->opt[OPT_GAMMA_VECTOR_B].cap &= ~SANE_CAP_INACTIVE;
+		    }
+		  break;
+		}
 	      /* The VM3575, VM3564, VM356A and VM6586 supports only a handful of resolution. Do that here.
 	       * Ugly! */
 	      if (dev->resolutions_list != NULL)
@@ -2626,15 +2985,16 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
        * in the SET WINDOWS command. */
       if (dev->val[OPT_PREVIEW].w == SANE_TRUE)
 	{
-	  /* for VM356A, no good 50 dpi scan possible */
+	  /* for VM356A, no good 50 dpi scan possible, now leave as */
 
 	  switch (dev->def->tecoref)
 	    {
 	    case TECO_VM356A:
+	    case TECO_VM6575:
 	      dev->x_resolution = 75;
 	      dev->y_resolution = 75;
 	      break;
-	      /* For VM3575, VM6575, VM656A, VM6586 etc until otherwise default value is used */
+	      /* For VM3575, VM656A, VM6586 etc until otherwise default value is used */
 	    default:
 	      dev->x_resolution = 50;
 	      dev->y_resolution = 50;
@@ -2735,7 +3095,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 
 	      int i;
 	      for (i = 0;
-		   dev->def->color_adjust[i].resolution != dev->x_resolution;
+		   dev->def->color_adjust[i].resolution != dev->y_resolution;
 		   i++);
 
 	      dev->color_adjust = &dev->def->color_adjust[i];
@@ -2842,7 +3202,7 @@ sane_start (SANE_Handle handle)
 
       switch (dev->def->tecoref)
 	{
-	case TECO_VM3564:
+	/* case TECO_VM3564: not for VM3564 */
 	case TECO_VM356A:
 /*--------------request sense for  first time loop---*/
 	  status = teco_request_sense (dev);
@@ -2975,22 +3335,19 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
     }
   while ((buf_offset != max_len) && dev->bytes_left);
 
-  DBG (DBG_info, "sane_read: leave, bytes_left=%d\n", dev->bytes_left);
+  DBG (DBG_info, "sane_read: leave, bytes_left=%ld\n", (long) dev->bytes_left);
 
   return SANE_STATUS_GOOD;
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (dev->scanning == SANE_FALSE)
     {
       return SANE_STATUS_INVAL;
@@ -3011,13 +3368,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
