@@ -2767,7 +2767,7 @@ check_rts8822l01h (struct usb_device *dev)
 		dev->descriptor.bDeviceClass);
       return 0;
     }
-  if (dev->descriptor.bcdUSB != 0x200)
+  if ((dev->descriptor.bcdUSB != 0x200)&&(dev->descriptor.bcdUSB != 0x110))
     {
       if (verbose > 2)
 	printf ("    this is not a RTS8822L-01H (bcdUSB = 0x%x)\n",
@@ -2882,7 +2882,7 @@ check_rts8822l01h (struct usb_device *dev)
       return 0;
     }
 
-  if ((data[0] & 0x01) != 0x01)
+  if (data == 0)
     {
       if (verbose > 2)
 	printf ("    Unexpected result from register 0xfe11 : 0x%0x%0x\n",
@@ -2960,9 +2960,11 @@ check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file)
   if (verbose > 2)
     {
       if (chip_name)
-	printf ("<This USB chip looks like a %s>\n\n", chip_name);
+	printf ("<This USB chip looks like a %s (result from %s)>\n\n",
+		chip_name, PACKAGE_STRING);
       else
-	printf ("<Couldn't determine the type of the USB chip>\n\n");
+	printf ("<Couldn't determine the type of the USB chip (result from %s)>\n\n",
+		PACKAGE_STRING);
     }
 
   return chip_name;
